﻿using FluentMigrator.Runner;
using Microsoft.Extensions.DependencyInjection;
using MySql.Data.MySqlClient;
using System;
using MyTrelloWeb.VCommon.Logging;

namespace MyTrelloWeb.App
{
    public class MyTrelloDbMigrator
    {
        public static void Run()
        {
            //Create database if no exist

            var builder = new MySqlConnectionStringBuilder(MyTrelloConfig.DefaultConnectionString);
            var dbName = builder.Database;
            builder.Remove("database");

            using (var conn = new MySqlConnection(builder.ConnectionString))
            {
                conn.Open();
                using (var cmd = conn.CreateCommand())
                {
                    cmd.CommandText = "SHOW DATABASES LIKE @" + nameof(dbName);
                    cmd.Parameters.Add(new MySqlParameter(nameof(dbName), dbName));

                    var dbCheckResult = (string)cmd.ExecuteScalar();

                    cmd.Parameters.Clear();
                    if (null == dbCheckResult || dbCheckResult.ToLower() != dbName.ToLower())
                    {
                        Logger.Info("Db no exist, try create...");
                        cmd.CommandText = $"CREATE DATABASE {dbName} CHARACTER SET utf8 COLLATE utf8_bin";
                        cmd.ExecuteNonQuery();
                        Logger.Info("Db created");
                    }
                }
            }

            // Run FluentMigrator

            var serviceProvider = CreateServices();

            // Put the database update into a scope to ensure
            // that all resources will be disposed.
            using (var scope = serviceProvider.CreateScope())
            {
                UpdateDatabase(scope.ServiceProvider);
            }
        }

        /// <summary>
        /// Configure the dependency injection services
        /// </summary>
        private static IServiceProvider CreateServices()
        {
            return new ServiceCollection()
                // Add common FluentMigrator services
                .AddFluentMigratorCore()
                .ConfigureRunner(rb => rb
                    // Add SQLite support to FluentMigrator
                    .AddMySql5()
                    // Set the connection string
                    .WithGlobalConnectionString(MyTrelloConfig.DefaultConnectionString)
                    // Define the assembly containing the migrations
                    .ScanIn(typeof(MyTrelloDbMigrator).Assembly).For.Migrations())
                // Enable logging to console in the FluentMigrator way
                .AddLogging(lb => lb.AddFluentMigratorConsole())
                // Build the service provider
                .BuildServiceProvider(false);
        }

        /// <summary>
        /// Update the database
        /// </summary>
        private static void UpdateDatabase(IServiceProvider serviceProvider)
        {
            // Instantiate the runner
            var runner = serviceProvider.GetRequiredService<IMigrationRunner>();

            // Execute the migrations
            runner.MigrateUp();
        }
    }
}
