﻿using MyTrelloWeb.VCommon.Json;
using MyTrelloWeb.VCommon.Logging;
using MyTrelloWeb.VCommon.Reflection;
using MyTrelloWeb.VCommon.VApplication;
using MyTrelloWeb.VCommon.VOpenApi.Docgen;
using System;
using System.Collections.Generic;
using System.Reflection;
using System.Web;

namespace MyTrelloWeb.VCommon.VOpenApi
{
    public abstract class ApiModuleBase : IHttpModule
    {
        protected virtual bool Enable => true;
        protected virtual bool EnableDocGen => true;
        protected string Title { get; }
        protected string Version { get; }
        protected string BaseApiPath { get; }

        public IReadOnlyDictionary<string, Type> Services { get; }

        private IReadOnlyDictionary<string, ApiBind> _apiRoutes;
        private string _openApiDoc;
        private IReadOnlyCollection<string> _schemes;

        /// <param name="baseApiPath">不带斜杠结尾</param>
        protected ApiModuleBase(string title, string version, string baseApiPath, IReadOnlyDictionary<string, Type> services, params string[] schemes)
        {
            _schemes = schemes;

            Title = title;
            Version = version;
            BaseApiPath = baseApiPath;
            Services = services;
        }

        public void Init(HttpApplication context)
        {
            if (false == Enable) return;
            try
            {
                //生成路由和调用绑定
                //暂定路由规则(不区分大小写): /<资源>/<方法>

                var apiRoutes = new Dictionary<string, ApiBind>();

                foreach (var service in Services)
                {
                    var resource = service.Key;
                    var interfaceType = service.Value;

                    if (false == interfaceType.IsInterface)
                    {
                        Logger.Warn("ApiModule: Not an interface type registered", new { resource, interfaceType });
                        continue;
                    }

                    var methods = interfaceType.GetPublicInstanceMethods();

                    foreach (var method in methods)
                    {
                        var rawRoute = $"/{resource}/{method.Name}";
                        var route = rawRoute.ToLower();
                        if (apiRoutes.ContainsKey(route))
                        {
                            Logger.Warn("ApiModule: Api method name already exist", route);
                            continue;
                        }

                        var parameterInfos = method.GetParameters();
                        var inputParams = parameterInfos;
                        if (1 < inputParams.Length)
                        {
                            Logger.Warn("ApiModule: Api method params more than 1", new { route, method });
                            continue;
                        }

                        apiRoutes[route] = new ApiBind(method, resource, method.Name, () => ResolveService(interfaceType), rawRoute);
                    }
                }

                _apiRoutes = apiRoutes;

                _openApiDoc = EnableDocGen
                   ? DocGenerator.Generate(Title, Version, BaseApiPath, _apiRoutes.Values, _schemes)
                   : VJsonSerializer.Serialize(new { docGen = false });

                //一切顺利才加挂请求处理事件,否则就当模块不存在
                context.BeginRequest += OnBeginRequest;
            }
            catch (Exception exception)
            {
                Logger.Fatal("ApiModule: init fail", exception);
            }
        }

        protected abstract object ResolveService(Type intrefaceType);

        protected virtual void OnBeginRequest(object sender, EventArgs eventArgs)
        {
            var requestPath = HttpContext.Current.Request.Path.ToLower();
            if (!requestPath.StartsWith(BaseApiPath)) return;

            var route = requestPath.Substring(BaseApiPath.Length);
            if ("/" == route) //根目录吐出文档
            {
                HttpContext.Current.Response.ContentType = "application/json";
                HttpContext.Current.Response.Write(_openApiDoc);
            }
            else if (_apiRoutes.TryGetValue(route, out var invokeBind))
            {
                HttpContext.Current.Response.ContentType = "application/json";

                try
                {
                    invokeBind.Invoke(HttpContext.Current.Request.InputStream, HttpContext.Current.Response.OutputStream);
                }
                catch (Exception exception)
                {
                    if (exception is TargetInvocationException) exception = exception.InnerException;

                    ErrorHandle(exception);
                }
            }
            else
            {
                HttpContext.Current.Response.StatusCode = 404;
                HttpContext.Current.Response.ContentType = "text/plain";
                HttpContext.Current.Response.Write("404 Not found");
            }

            HttpContext.Current.ApplicationInstance.CompleteRequest();
        }

        protected virtual void ErrorHandle(Exception error)
        {
            //400:请求参数格式错误 VApiArgumentException
            //401:未授权(未登录) VApplicationAuthException 1
            //403:拒绝访问(已登录但没有权限) VApplicationAuthException 2
            //422:模型检查失败 VApplicationModelValidationException
            //500:服务器内部错误 *其他

            HttpContext.Current.Response.ContentType = "application/json";
            if (error is VApiArgumentException apiArgumentException)
            {
                HttpContext.Current.Response.StatusCode = 400;
                HttpContext.Current.Response.Write(VJsonSerializer.Serialize(new { apiArgumentException.Message }));
            }
            else if (error is VApplicationModelValidationException modelValidationException)
            {
                HttpContext.Current.Response.StatusCode = 422;
                HttpContext.Current.Response.Write(VJsonSerializer.Serialize(new { modelValidationException.Message, modelValidationException.Results }));
            }
            else if (error is VApplicationAuthException authException)
            {
                HttpContext.Current.Response.StatusCode = authException.Reason == AuthReason.AuthRequired ? 401 : 403;
                HttpContext.Current.Response.Write(VJsonSerializer.Serialize(new { authException.Message }));
            }
            else
            {
                HttpContext.Current.Response.StatusCode = 500;

                object response;
                if (false == HttpContext.Current.IsDebuggingEnabled)
                {
                    response = VJsonSerializer.Serialize(new { Message = "Server internal error" });
                }
                else
                {
                    try
                    {
                        response = VJsonSerializer.Serialize(error);
                    }
                    catch (Exception exception)
                    {
                        Logger.Error("ApiModule:Exception json serializer fail", exception);
                        response = VJsonSerializer.Serialize(new { Message = error.ToString() });
                    }
                }

                HttpContext.Current.Response.Write(response);
            }
            HttpContext.Current.Response.TrySkipIisCustomErrors = true; // 禁用 IIS 的自定义错误页面
        }

        public virtual void Dispose()
        {
            if (false == Enable) return;
            //释放资源,如果有
        }
    }
}
